@@ -8,7 +8,7 @@ use SVK::Target;
 use Pod::Simple::Text ();
 use Pod::Simple::SimpleTree ();
 use File::Find ();
-use SVK::Util qw( abs_path $SEP IS_WIN32 );
+use SVK::Util qw( get_prompt abs_path is_uri catdir $SEP IS_WIN32 HAS_SVN_MIRROR );
 use SVK::I18N;
 
 =head1 NAME
@@ -36,29 +36,44 @@ information is displayed instead.
 
 =cut
 
-my %alias = qw( co checkout
-		up update
-		blame annotate
-		ci commit
-		del delete
-		rm delete
-		pg propget
-		ps propset
-		pe propedit
-		pl proplist
-		cp copy
-		mv move
-		ren move
-		rename move
-		mi mirror
-		sm smerge
-		sy sync
-		desc describe
-		st status
-		stat status
-		ver version
-		ls list
-	      );
+my %alias = qw( ann		annotate
+                blame		annotate
+                praise		annotate
+		co		checkout
+		cm		cmerge
+		ci		commit
+		cp		copy
+		del		delete
+		remove		delete
+		rm		delete
+		depot		depotmap
+		desc		describe
+		di		diff
+                h               help
+                ?               help
+		ls		list
+		mi		mirror
+		mv		move
+		ren		move
+		rename	    	move
+		pd		propdel
+		pdel		propdel
+		pe		propedit
+		pedit		propedit
+		pg		propget
+		pget		propget
+		pl		proplist
+		plist		proplist
+		ps		propset
+		pset		propset
+		sm		smerge
+		st		status
+		stat		status
+		sw		switch
+		sy		sync
+		up		update
+		ver		version
+	    );
 
 =head3 new ($xd)
 
@@ -82,11 +97,13 @@ instance of it, populated with C<$xd>.  Command aliases are handled here.
 sub get_cmd {
     my ($pkg, $cmd, $xd) = @_;
     die "Command not recognized, try $0 help.\n"
-	unless $cmd =~ m/^[a-z]+$/;
+	unless $cmd =~ m/^[?a-z]+$/;
     $pkg = join('::', 'SVK::Command', _cmd_map ($cmd));
-    unless (eval "require $pkg; 1" && UNIVERSAL::can($pkg, 'run')) {
-	$pkg =~ s|::|/|g;
-	warn $@ if $@ && exists $INC{"$pkg.pm"};
+    my $file = "$pkg.pm";
+    $file =~ s!::!/!g;
+
+    unless (eval {require $file; 1} and $pkg->can('run')) {
+	warn $@ if $@ and exists $INC{$file};
 	die "Command not recognized, try $0 help.\n";
     }
     $pkg->new ($xd);
@@ -113,7 +130,6 @@ arguments are expanded for shell globbing with C<File::Glob::bsd_glob>.
 sub invoke {
     my ($pkg, $xd, $cmd, $output, @args) = @_;
     my ($help, $ofh, $ret);
-    local @ARGV = @args;
     my $pool = SVN::Pool->new_default;
     $ofh = select $output if $output;
     my $error;
@@ -121,29 +137,30 @@ sub invoke {
 	$error = $_[0];
 	SVN::Error::croak_on_error (@_);
     };
+
+    local $@;
     eval {
 	$cmd = get_cmd ($pkg, $cmd, $xd);
 	$cmd->{svnconfig} = $xd->{svnconfig} if $xd;
-	die loc ("Unknown options.\n")
-	    unless GetOptions ('h|help' => \$help, _opt_map($cmd, $cmd->options));
+	$cmd->getopt (\@args, 'h|help|?' => \$help);
 
 	# Fake shell globbing on Win32 if we are called from main
 	if (IS_WIN32 and caller(1) eq 'main') {
 	    require File::Glob;
-	    @ARGV = map {
+	    @args = map {
 		/[?*{}\[\]]/
 		    ? File::Glob::bsd_glob($_, File::Glob::GLOB_NOCHECK())
 		    : $_
-	    } @ARGV;
+	    } @args;
 	}
 
-	if ($help || !(@args = $cmd->parse_arg(@ARGV))) {
+	if ($help || !(@args = $cmd->parse_arg(@args))) {
 	    select STDERR unless $output;
 	    $cmd->usage;
 	}
 	else {
+	    $cmd->msg_handler ($SVN::Error::FS_NO_SUCH_REVISION);
 	    eval { $cmd->lock (@args); $ret = $cmd->run (@args) };
-	    print STDERR "======>[$@]\n" if $ENV{DEBUG};
 	    $xd->unlock if $xd;
 	    die $@ if $@;
 	}
@@ -156,6 +173,20 @@ sub invoke {
     select $ofh if $ofh
 }
 
+=head3 getopt ($argv, %opt)
+
+Takes a arrayref of argv for run getopt for the command, with
+additional %opt getopt options.
+
+=cut
+
+sub getopt {
+    my ($self, $argv, %opt) = @_;
+    local *ARGV = $argv;
+    die loc ("Unknown options.\n")
+	unless GetOptions (%opt, $self->_opt_map ($self->options));
+}
+
 =head2 Instance Methods
 
 C<SVK::Command-E<gt>invoke> loads the corresponding class
@@ -275,6 +306,103 @@ sub arg_condensed {
     return $target;
 }
 
+=head3 arg_uri_maybe ($arg)
+
+Argument might be a URI or a depotpath.  If it is a URI, try to find it
+at or under one of currently mirrored paths.  If not found, prompts the
+user to mirror and sync it.
+
+=cut
+
+sub arg_uri_maybe {
+    my ($self, $arg) = @_;
+
+    is_uri($arg) or return $self->arg_depotpath($arg);
+    HAS_SVN_MIRROR or die loc("cannot load SVN::Mirror");
+
+    require URI;
+    my $uri = URI->new("$arg/")->canonical or die loc("%1 is not a valid URI.\n", $arg);
+    my $map = $self->{xd}{depotmap};
+    foreach my $depot (sort keys %$map) {
+        my $repos = ($self->{xd}->find_repos ("/$depot/", 1))[2];
+	foreach my $path ( SVN::Mirror::list_mirror ($repos) ) {
+	    my $m = SVN::Mirror->new (
+                repos => $repos,
+                get_source => 1,
+                target_path => $path,
+            );
+
+            my $rel_uri = $uri->rel(URI->new("$m->{source}/")->canonical) or next;
+            next if $rel_uri->eq($uri);
+            next if $rel_uri =~ /^\.\./;
+
+            my $depotpath = catdir('/', $depot, $path, $rel_uri);
+            $depotpath = "/$depotpath" if !length($depot);
+            return $self->arg_depotpath($depotpath);
+	}
+    }
+
+    print loc("New URI encountered: %1\n", $uri);
+
+    my $depots = join('|', map quotemeta, sort keys %$map);
+    my ($base_uri, $rel_uri);
+
+    {
+        my $base = get_prompt(
+            loc("Choose a base URI to mirror from (press enter to use the full URI): ", $uri),
+            qr/^(?:[A-Za-z][-+.A-Za-z0-9]*:|$)/
+        );
+        if (!length($base)) {
+            $base_uri = $uri;
+            $rel_uri = '';
+            last;
+        }
+
+        $base_uri = URI->new("$base/")->canonical;
+
+        $rel_uri = $uri->rel($base_uri);
+        next if $rel_uri->eq($uri);
+        next if $rel_uri =~ /^\.\./;
+        last;
+    }
+
+    my $path = get_prompt(
+        loc("Name a depot path for this mirror (under //mirror/ if no leading '/'): "),
+        qr{^(?:/(?:$depots)/)?\w+},
+    );
+    $path = "//mirror/$path" unless $path =~ m!^/!;
+
+    my $target = $self->arg_depotpath($path);
+    require SVK::Command::Mirror;
+    my $mirror = SVK::Command::Mirror->new;
+    $mirror->run($target, $base_uri);
+
+    print loc("Synchronizing the mirror for the first time:\n");
+    print loc("  a        : Retrieve all revisions (default)\n");
+    print loc("  h        : Only the most recent revision\n");
+    print loc("  -count   : At most 'count' recent revisions\n");
+    print loc("  revision : Start from the specified revision\n");
+
+    my $answer = lc(get_prompt(
+        loc("a)ll, h)ead, -count, revision? [a] "),
+        qr(^[ah]?|^-?\d+$)
+    ));
+    $answer = 'a' unless length $answer;
+
+    require SVK::Command::Sync;
+    my $sync = SVK::Command::Sync->new;
+    $sync->{skip_to} = (
+        ($answer eq 'a') ? undef :
+        ($answer eq 'h') ? 'HEAD-1' :
+        ($answer < 0)    ? "HEAD$answer" :
+                           $answer
+    );
+    $sync->run ($target);
+
+    my $depotpath = "$target->{depotpath}/$rel_uri";
+    return $self->arg_depotpath($depotpath);
+}
+
 =head3 arg_co_maybe ($arg)
 
 Argument might be a checkout path or a depotpath.
@@ -283,6 +411,9 @@ Argument might be a checkout path or a depotpath.
 
 sub arg_co_maybe {
     my ($self, $arg) = @_;
+
+    $arg = $self->arg_uri_maybe($arg)->{depotpath} if is_uri($arg);
+
     my $rev = $arg =~ s/\@(\d+)$// ? $1 : undef;
     my ($repospath, $path, $copath, $cinfo, $repos) =
 	$self->{xd}->find_repos_from_co_maybe ($arg, 1);
@@ -325,6 +456,7 @@ Argument is a depotpath, including the slashes and depot name.
 
 sub arg_depotpath {
     my ($self, $arg) = @_;
+
     my $rev = $arg =~ s/\@(\d+)$// ? $1 : undef;
     my ($repospath, $path, $repos) = $self->{xd}->find_repos ($arg, 1);
 
@@ -338,6 +470,24 @@ sub arg_depotpath {
 	);
 }
 
+=head3 arg_depotroot ($arg)
+
+Argument is a depot root, or a checkout path that needs to be resolved
+into a depot root.
+
+=cut
+
+sub arg_depotroot {
+    my ($self, $arg) = @_;
+
+    local $@;
+    $arg = eval { $self->arg_co_maybe ($arg || '')->new (path => '/') }
+           || $self->arg_depotpath ("//");
+    $arg->as_depotpath;
+
+    return $arg;
+}
+
 =head3 arg_depotname ($arg)
 
 Argument is a name of depot. such as '' or 'test' that is being used
@@ -495,7 +645,7 @@ sub msg_handler {
     my ($self, $err, $msg) = @_;
     $self->add_handler
 	($err, sub {
-	     print $_[0]->expanded_message."\n$msg\n";
+	     print $_[0]->expanded_message."\n".($msg ? "$msg\n" : '');
 	 });
 }
 
