@@ -8,24 +8,37 @@ use SVK::I18N;
 
 sub options {
     ($_[0]->SUPER::options,
-     'force'	    => 'force'
+     'f|from-checkout|force'    => 'from_checkout',
+     't|to-checkout'	        => 'to_checkout',
     )
 }
 sub parse_arg {
     my $self = shift;
-    my @arg = @_;
-    $arg[1] = '' if $#arg < 1;
+    my @arg = @_ or return;
+
+    return if @arg > 2;
+    unshift @arg, '' while @arg < 2;
+
+    if (eval { $self->{xd}->find_repos($arg[1]); 1 }) {
+        # Reorder to put DEPOTPATH before PATH
+        @arg[0,1] = @arg[1,0];
+    }
 
     return ($self->arg_depotpath ($arg[0]), $self->arg_path ($arg[1]));
 }
 
 sub lock {
     my ($self, $target, $source) = @_;
-    return $self->lock_none
-	unless $self->{xd}{checkout}->get ($source)->{depotpath};
+    unless ($self->{xd}{checkout}->get ($source)->{depotpath}) {
+	return $self->{to_checkout} ? $self->{xd}->lock ($source)
+	    : $self->lock_none;
+    }
     $source = $self->arg_copath ($source);
-    ($self->{force} && $target->{path} eq $source->{path}) ?
-	$self->lock_target ($source) : $self->lock_none;
+    die loc("Import source (%1) is a checkout path; use --from-checkout.\n", $source->{copath})
+	unless $self->{from_checkout};
+    die loc("Import path (%1) is different from the copath (%2)\n", $target->{path}, $source->{path})
+	unless $source->{path} eq $target->{path};
+    $self->lock_target ($source);
 }
 
 sub mkpdir {
@@ -46,6 +59,7 @@ sub mkpdir {
 
 sub run {
     my ($self, $target, $copath) = @_;
+    return unless $self->check_mirrored_path ($target) || $self->{from_checkout};
 
     my $fs = $target->{repos}->fs;
     my $yrev = $fs->youngest_rev;
@@ -60,22 +74,13 @@ sub run {
 	$root = $fs->revision_root ($yrev);
     }
 
-    if (exists $self->{xd}{checkout}->get ($copath)->{depotpath}) {
-	$self->{is_checkout}++;
-	die loc("Import source cannot be a checkout path")
-	    unless $self->{force};
-	# XXX: check if anchor matches
-	my (undef, $path) = $self->{xd}->find_repos_from_co ($copath, 0);
-	die loc("Import path (%1) is different from the copath (%2)\n", $target->{path}, $path)
-	    unless $path eq $target->{path};
-
-    }
-    else {
+    unless (exists $self->{xd}{checkout}->get ($copath)->{depotpath}) {
 	$self->{xd}{checkout}->store
 	    ($copath, {depotpath => $target->{depotpath},
 		       '.newprop' => undef,
 		       '.conflict' => undef,
 		       revision => $target->{revision}});
+        delete $self->{from_checkout};
     }
 
     $self->get_commit_message () unless $self->{check_only};
@@ -85,7 +90,16 @@ sub run {
 	      print loc("Directory %1 imported to depotpath %2 as revision %3.\n",
 			$copath, $target->{depotpath}, $yrev);
 
-	      if ($self->{is_checkout}) {
+	      if ($self->{to_checkout}) {
+                  $self->{xd}{checkout}->store_recursively (
+                      $copath, {
+                          depotpath => $target->{depotpath},
+                          revision => $yrev,
+                          $self->_schedule_empty,
+                      }
+                  );
+              }
+              elsif ($self->{from_checkout}) {
 		  $self->committed_import ($copath)->($yrev);
 	      }
 	      else {
@@ -100,7 +114,6 @@ sub run {
     $self->run_delta ($target->new (copath => $copath), $root, $editor, %cb);
 }
 
-
 1;
 
 __DATA__
@@ -111,14 +124,18 @@ SVK::Command::Import - Import directory into depot
 
 =head1 SYNOPSIS
 
+ import [PATH] DEPOTPATH
+
+ # You may also list the target part first:
  import DEPOTPATH [PATH]
 
 =head1 OPTIONS
 
  -m [--message] arg     : specify commit message ARG
  -C [--check-only]      : try operation but make no changes
- -s [--sign]            : sign this change
- --force                : force import from a checkout path
+ -S [--sign]            : sign this change
+ -f [--from-checkout]   : import from a checkout path
+ -t [--to-checkout]     : turn the source into a checkout path
 
 =head1 AUTHORS
 
