@@ -55,7 +55,7 @@ SVK::Command::Mkdir - Create a versioned directory
 
  -m [--message] arg     : specify commit message ARG
  -C [--check-only]      : try operation but make no changes
- -s [--sign]            : sign this change
+ -S [--sign]            : sign this change
 
 =head1 AUTHORS
 
