@@ -2,19 +2,22 @@ package SVK::Command::Propget;
 use strict;
 our $VERSION = $SVK::VERSION;
 
-use base qw( SVK::Command );
+use base qw( SVK::Command::Proplist );
 use SVK::XD;
 
 sub options {
-    ('strict'	=> 'strict',
+    ('strict' => 'strict',
+     'R|recursive' => 'recursive',
+     'r|revision=i' => 'rev',
+     'revprop' => 'revprop',
     );
 }
 
 sub parse_arg {
     my ($self, @arg) = @_;
     return if @arg < 1;
-    push @arg, '.' if @arg == 1;
-    return ($arg[0], map {$self->arg_co_maybe ($_)} @arg[1..$#arg]);
+    push @arg, '' if @arg == 1;
+    return ($arg[0], map { $self->_arg_revprop ($_) } @arg[1..$#arg]);
 }
 
 sub lock { $_[0]->lock_none }
@@ -23,7 +26,7 @@ sub run {
     my ($self, $pname, @targets) = @_;
 
     foreach my $target (@targets) {
-        my $proplist = $self->{xd}->do_proplist ($target);
+        my $proplist = $self->_proplist($target);
         exists $proplist->{$pname} or next;
 
         print $target->path, ' - ' if @targets > 1 and !$self->{strict};
@@ -48,6 +51,9 @@ SVK::Command::Propget - Display a property on path
 
 =head1 OPTIONS
 
+ -R [--recursive]       : descend recursively
+ -r [--revision] arg    : act on revision ARG instead of the head revision
+ --revprop              : operate on a revision property (use with -r)
  --strict               : do not print an extra newline at the end of the
                           property values; when there are multiple paths
                           involved, do not prefix path names before values
